!---------------------------------------------------------------------!
!                                PHAML                                !
!                                                                     !
! The Parallel Hierarchical Adaptive MultiLevel code for solving      !
! linear elliptic partial differential equations of the form          !
! (PUx)x + (QUy)y + RU = F on 2D polygonal domains with mixed         !
! boundary conditions, and eigenvalue problems where F is lambda*U.   !
!                                                                     !
! PHAML is public domain software.  It was produced as part of work   !
! done by the U.S. Government, and is not subject to copyright in     !
! the United States.                                                  !
!                                                                     !
!     William F. Mitchell                                             !
!     Mathematical and Computational Sciences Division                !
!     National Institute of Standards and Technology                  !
!     william.mitchell@nist.gov                                       !
!     http://math.nist.gov/phaml                                      !
!                                                                     !
!---------------------------------------------------------------------!

module patch_problem

!----------------------------------------------------
! This is a dummy version of module patch_problem, to use in situations that
! do not support the patch problems.  All uses of dummy arguments are
! just to avoid warnings from compilers that warn about unused dummy arguments.
!----------------------------------------------------

!----------------------------------------------------
! Other modules used:

use global, only : my_real, ierr, PHAML_INTERNAL_ERROR
use message_passing, only : fatal
use gridtype_mod, only : grid_type
use linsystype_mod, only : solver_options

implicit none
private

public patch_type, create_patch, destroy_patch, init_patch, h_refine_patch, &
       p_refine_patch, save_p_patch, restore_p_patch, &
       set_patch_core_children_p, create_patch_linear_system, &
       destroy_patch_linear_system, solve_patch_linear_system, &
       get_patch_linear_system_dof, norm_patch_soln, print_patch_grid, &
       print_patch_linear_system

type patch_type
   private
   integer :: dummy
end type patch_type

contains

!          ------------
subroutine create_patch(patch)
!          ------------
type(patch_type), intent(out) :: patch
ierr = PHAML_INTERNAL_ERROR
call fatal("Called dummy version of create_patch")
stop
end subroutine create_patch

!          -------------
subroutine destroy_patch(patch)
!          -------------
type(patch_type), intent(inout) :: patch
ierr = PHAML_INTERNAL_ERROR
call fatal("Called dummy version of destroy_patch")
stop
end subroutine destroy_patch

!          ----------
subroutine init_patch(grid,patch,core,include_mate,halo_size, &
                      include_vert_neigh,boundary_condition,eigen)
!          ----------
type(grid_type), intent(inout) :: grid
type(patch_type), intent(inout) :: patch
integer, intent(in) :: core
logical, intent(in) :: include_mate
integer, intent(in) :: halo_size
logical, intent(in) :: include_vert_neigh
integer, intent(in) :: boundary_condition
integer, intent(in), optional :: eigen
ierr = PHAML_INTERNAL_ERROR
call fatal("Called dummy version of init_patch")
stop
end subroutine init_patch

!          --------------
subroutine h_refine_patch(patch)
!          --------------
type(patch_type), intent(inout) :: patch
ierr = PHAML_INTERNAL_ERROR
call fatal("Called dummy version of h_refine_patch")
stop
end subroutine h_refine_patch

!          --------------
subroutine p_refine_patch(patch,rule)
!          --------------
type(patch_type), intent(inout) :: patch
integer, intent(in) :: rule
ierr = PHAML_INTERNAL_ERROR
call fatal("Called dummy version of p_refine_patch")
stop
end subroutine p_refine_patch

!          ------------
subroutine save_p_patch(patch)
!          ------------
type(patch_type), intent(inout) :: patch
ierr = PHAML_INTERNAL_ERROR
call fatal("Called dummy version of save_p_patch")
stop
end subroutine save_p_patch

!          ---------------
subroutine restore_p_patch(patch)
!          ---------------
type(patch_type), intent(inout) :: patch
ierr = PHAML_INTERNAL_ERROR
call fatal("Called dummy version of restore_p_patch")
stop
end subroutine restore_p_patch

!          -------------------------
subroutine set_patch_core_children_p(patch,deg,rule)
!          -------------------------
type(patch_type), intent(inout) :: patch
integer, intent(in) :: deg(2)
integer, intent(in) :: rule
ierr = PHAML_INTERNAL_ERROR
call fatal("Called dummy version of set_patch_core_children_p")
stop
end subroutine set_patch_core_children_p

!          --------------------------
subroutine create_patch_linear_system(patch,grid,resid_prob,just_rhs, &
                                      solver_control,loc_bconds_a,loc_bconds_s)
!          --------------------------
type(patch_type), intent(inout) :: patch
type(grid_type), intent(in) :: grid
logical, intent(in) :: resid_prob
logical, intent(in) :: just_rhs
type(solver_options), intent(in), optional :: solver_control
interface
   subroutine loc_bconds_a(x,y,z,bmark,itype,c,rs,extra_rs)
   use global
   real(my_real), intent(in) :: x(:),y(:),z(:)
   integer, intent(in) :: bmark
   integer, intent(out) :: itype(:)
   real(my_real), intent(out) :: c(:,:,:),rs(:,:)
   real(my_real), intent(out), optional :: extra_rs(:,:,:)
   end subroutine loc_bconds_a
   subroutine loc_bconds_s(x,y,z,bmark,itype,c,rs)
   use global
   real(my_real), intent(in) :: x,y,z
   integer, intent(in) :: bmark
   integer, intent(out) :: itype(:)
   real(my_real), intent(out) :: c(:,:),rs(:)
   end subroutine loc_bconds_s
end interface
optional :: loc_bconds_a, loc_bconds_s
ierr = PHAML_INTERNAL_ERROR
call fatal("Called dummy version of create_patch_linear_system")
stop
end subroutine create_patch_linear_system

!          ----------------------------
subroutine destroy_patch_linear_system(patch)
!          ----------------------------
type(patch_type), intent(inout) :: patch
ierr = PHAML_INTERNAL_ERROR
call fatal("Called dummy version of destroy_patch_linear_system")
stop
end subroutine destroy_patch_linear_system

!          -------------------------
subroutine solve_patch_linear_system(patch)
!          -------------------------
type(patch_type), intent(inout) :: patch
ierr = PHAML_INTERNAL_ERROR
call fatal("Called dummy version of solve_patch_linear_system")
stop
end subroutine solve_patch_linear_system

!        ---------------------------
function get_patch_linear_system_dof(patch,nodirich)
!        ---------------------------
type(patch_type), intent(inout) :: patch
logical, optional, intent(in) :: nodirich
integer :: get_patch_linear_system_dof
if (present(nodirich)) then
   patch%dummy = 1
endif
get_patch_linear_system_dof = 0
ierr = PHAML_INTERNAL_ERROR
call fatal("Called dummy version of get_patch_linear_system_dof")
stop
end function get_patch_linear_system_dof

!          ---------------
subroutine norm_patch_soln(patch,energy,Linf,L2,nodirich)
!          ---------------
type(patch_type), intent(in) :: patch
real(my_real), intent(out), optional :: energy(:),Linf(:,:),L2(:,:)
logical, intent(in), optional :: nodirich
ierr = PHAML_INTERNAL_ERROR
call fatal("Called dummy version of norm_patch_soln")
stop
end subroutine norm_patch_soln

!          ----------------
subroutine print_patch_grid(patch)
!          ----------------
type(patch_type), intent(in) :: patch
ierr = PHAML_INTERNAL_ERROR
call fatal("Called dummy version of print_patch_grid")
stop
end subroutine print_patch_grid

!          -------------------------
subroutine print_patch_linear_system(patch)
!          -------------------------
type(patch_type), intent(in) :: patch
ierr = PHAML_INTERNAL_ERROR
call fatal("Called dummy version of print_patch_linear_system")
stop
end subroutine print_patch_linear_system

end module patch_problem
